//-------------------------------------------------------------------------//
// GetKernel32Base.exe
//-------------------------------------------------------------------------//
// A popular method of retrieving the kernel32.dll base address relies on
// parsing the PEB for the Ldr.InInitializationOrderModuleList structure.
// The second entry in this list is allways kernel32.dll except when run
// on Windows 7 (rc1) where the second entry is kernelbase.dll (Due to 
// the new MinWin kernel structure).
//
// An alternative method of retrieving the kernel32.dll base address relies
// on parsing the PEB for the Ldr.InMemoryOrderModuleList structure. The 
// third entry in this list is allways kernel32.dll (the first being the
// main module and the second being ntdll.dll). This method has been tested
// and works correctly on the following:
//
//     Microsoft Windows [Version 5.0.2195] Service Pack 4 (Windows 2000)
//     Microsoft Windows [Version 5.1.2600] Service Pack 2 (Windows XP)
//     Microsoft Windows [Version 5.1.2600] Service Pack 3 (Windows XP)
//     Microsoft Windows [Version 5.2.3790] Service Pack 2 (Windows 2003)
//     Microsoft Windows [Version 6.0.6001] Service Pack 1 (Windows Vista)
//     Microsoft Windows [Version 6.0.6001] Service Pack 1 (Windows 2008)
//     Microsoft Windows [Version 6.1.7100]                (Windows 7 RC1)
//
// A third and more robust method has been added. It works by searching the 
// InMemoryOrder module list for the kernel32 module using a hash of the 
// module name for comparison. It also normalises the module name to uppercase
// as some systems store module names in uppercase and some in lowercase.
//-------------------------------------------------------------------------//
// Stephen Fewer of Harmony Security (www.harmonysecurity.com).
//-------------------------------------------------------------------------//
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
//-------------------------------------------------------------------------//
int main( void )
{
	OSVERSIONINFOEX VersionInfoEx = { 0 };
	DWORD dwBaseAddressA = 0;
	DWORD dwBaseAddressB = 0;
	DWORD dwBaseAddressC = 0;
	DWORD dwBaseAddressD = 0;

	VersionInfoEx.dwOSVersionInfoSize = sizeof( OSVERSIONINFOEX );

	GetVersionEx( (LPOSVERSIONINFO)&VersionInfoEx );

	if( VersionInfoEx.dwPlatformId != VER_PLATFORM_WIN32_NT )
		printf( "Warning: Not an NT based platform, YMMV!\n" );

	printf( "-------------------------------------------------------------\n" );
	printf( "Microsoft Windows [Version %d.%d.%d] %s\n", VersionInfoEx.dwMajorVersion, VersionInfoEx.dwMinorVersion, VersionInfoEx.dwBuildNumber, VersionInfoEx.szCSDVersion );
	printf( "-------------------------------------------------------------\n" );
	printf( "GetModuleHandleA( \"kernelbase.dll\" ) = 0x%08X\n", (DWORD)GetModuleHandleA( "kernelbase.dll" ) );
	printf( "GetModuleHandleA( \"kernel32.dll\" )   = 0x%08X\n", (DWORD)GetModuleHandleA( "kernel32.dll" ) );
	printf( "GetModuleHandleA( \"ntdll.dll\" )      = 0x%08X\n", (DWORD)GetModuleHandleA( "ntdll.dll" ) );
	printf( "-------------------------------------------------------------\n" );

	// Get the actual kernel32 base address...
	dwBaseAddressA = (DWORD)GetModuleHandleA( "kernel32.dll" );

	// Try the first method...
	__asm
	{
		xor ebx, ebx             ; // clear ebx
		mov ebx, fs:[ 0x30 ]     ; // get a pointer to the PEB
		mov ebx, [ ebx + 0x0C ]  ; // get PEB->Ldr
		mov ebx, [ ebx + 0x1C ]  ; // get PEB->Ldr.InInitializationOrderModuleList.Flink (1st entry)
		mov ebx, [ ebx ]         ; // get the next entry (2nd entry)
		mov ebx, [ ebx + 0x08 ]  ; // get the 2nd entries base address (kernel32.dll)

		mov dwBaseAddressB, ebx
	}

	printf( "Method 1: Kernel32 Base Address = 0x%08X (Correct: %s)\n", dwBaseAddressB, (dwBaseAddressA==dwBaseAddressB?"YES":"NO!") );

	// Try the second method...
	__asm
	{
		xor ebx, ebx               ; // clear ebx
		mov ebx, fs:[ 0x30 ]       ; // get a pointer to the PEB
		mov ebx, [ ebx + 0x0C ]    ; // get PEB->Ldr
		mov ebx, [ ebx + 0x14 ]    ; // get PEB->Ldr.InMemoryOrderModuleList.Flink (1st entry)
		mov ebx, [ ebx ]           ; // get the next entry (2nd entry)
		mov ebx, [ ebx ]           ; // get the next entry (3rd entry)
		mov ebx, [ ebx + 0x10 ]    ; // get the 3rd entries base address (kernel32.dll)

		mov dwBaseAddressC, ebx
	}

	printf( "Method 2: Kernel32 Base Address = 0x%08X (Correct: %s)\n", dwBaseAddressC, (dwBaseAddressA==dwBaseAddressC?"YES":"NO!") );
	
	// Try the third method...
	__asm
	{
		cld                       ; // clear the direction flag for the loop
		xor edx, edx              ; // zero edx

		mov edx, fs:[ 0x30 ]      ; // get a pointer to the PEB
		mov edx, [ edx+0x0C ]     ; // get PEB->Ldr
		mov edx, [ edx+0x14 ]     ; // get the first module from the InMemoryOrder module list
	next_mod:
		mov esi, [ edx+0x28 ]     ; // get pointer to modules name (unicode string)
		mov ecx, 24               ; // set ecx to this length for the loop
		xor edi, edi              ; // clear edi which will store the hash of the module name
	loop_modname:
		xor eax, eax              ; // clear eax
		lodsb                     ; // read in the next byte of the name
		cmp al, 'a'               ; // some versions of Windows use lower case module names
		jl not_lowercase
		sub al, 0x20              ; // if so normalise to uppercase
	not_lowercase:
		ror edi, 13               ; // rotate left our hash value
		add edi, eax              ; // add the next byte of the name
		loop loop_modname         ; // loop untill we have read enough
		cmp edi, 0x6A4ABC5B       ; // compare the hash with that of KERNEL32.DLL
		mov ebx, [ edx+0x10 ]     ; // get this modules base address
		mov edx, [ edx ]          ; // get the next module
		jne next_mod              ; // if it doesnt match, process the next module
		  
		mov dwBaseAddressD, ebx   ; // when we get here EBX is the kernel32 base (or change to suit).
	}

	printf( "Method 3: Kernel32 Base Address = 0x%08X (Correct: %s)\n", dwBaseAddressD, (dwBaseAddressA==dwBaseAddressD?"YES":"NO!") );

	printf( "-------------------------------------------------------------" );

	return 0;
}
//-------------------------------------------------------------------------//